import io, struct
from typing import List

from dataclasses import dataclass

TYPES = []


@dataclass()
class Byte_Type:
    id: int = 0
    c: str = ''
    len: int = 0

    def __init__(self, id: int, c: str, len: int):
        TYPES.insert(id, self)
        self.id = id
        self.c = c
        self.len = len

    def is_list(self):
        return self.id > 10


T_Bool = Byte_Type(1, '?', 1)
T_Byte = Byte_Type(2, 'b', 1)
T_Short = Byte_Type(3, 'h', 2)
T_Int = Byte_Type(4, 'i', 4)
T_Float = Byte_Type(5, 'f', 4)

L_Bool = Byte_Type(11, '?', 1)
L_Byte = Byte_Type(12, 'b', 1)
L_Short = Byte_Type(13, 'h', 2)
L_Int = Byte_Type(14, 'i', 4)
L_Float = Byte_Type(15, 'f', 4)


class Buffer():
    def __init__(self, buf=None):
        self._buf = io.BytesIO() if not buf else buf
        self._external = not buf
        self._buf.writable()

    def writes(self, fmt, *values):
        self._buf.write(struct.pack(fmt, *values))
        return self

    def reads(self, fmt, v_len):
        content = self._buf.read(v_len)
        if not content: return None
        return struct.unpack(fmt, content)

    def write(self, t: Byte_Type, value):
        self._buf.write(struct.pack(t.c, value))
        return self

    def write_bool(self, value):
        return self.write(T_Bool, value)

    def write_byte(self, value):
        return self.write(T_Byte, value)

    def write_short(self, value):
        return self.write(T_Short, value)

    def write_int(self, value):
        return self.write(T_Int, value)

    def write_float(self, value):
        return self.write(T_Float, value)

    def write_list(self, t: Byte_Type, values):
        self._buf.write(struct.pack(t.c * len(values), *values))
        return self

    def write_bools(self, values):
        return self.write_list(T_Bool, values)

    def write_bytes(self, values):
        return self.write_list(T_Byte, values)

    def write_shorts(self, values):
        return self.write_list(T_Short, values)

    def write_ints(self, values):
        return self.write_list(T_Int, values)

    def write_floats(self, values):
        return self.write_list(T_Float, values)

    def read(self, t: Byte_Type):
        content = self._buf.read(t.len)
        if not content: return None
        return struct.unpack(t.c, content)[0]

    def read_bool(self) -> bool:
        return self.read(T_Bool)

    def read_byte(self) -> int:
        return self.read(T_Byte)

    def read_short(self) -> int:
        return self.read(T_Short)

    def read_int(self) -> int:
        return self.read(T_Int)

    def read_float(self) -> float:
        return self.read(T_Float)

    def read_list(self, t: Byte_Type, length=1) -> List:
        content = self._buf.read(t.len * length)
        if not content: return None
        return list(struct.unpack(t.c * length, content))

    def read_bools(self, length=1) -> List[bool]:
        return self.read_list(T_Bool, length)

    def read_bytes(self, length=1) -> List[int]:
        return self.read_list(T_Byte, length)

    def read_shorts(self, length=1) -> List[int]:
        return self.read_list(T_Short, length)

    def read_ints(self, length=1) -> List[int]:
        return self.read_list(T_Int, length)

    def read_floats(self, length=1) -> List[float]:
        return self.read_list(T_Int, length)

    def seek(self, position=0):
        self._buf.seek(position)
        return self

    def write_buffer(self, buffer):
        self._buf.write(buffer.get_buffer())
        return self

    def get_buffer(self):
        return self._buf.getbuffer()

    def close(self):
        if not self._external and self._buf and not self._buf.closed: self._buf.close()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()


class ObjectListSerializer:
    def __init__(self, obj_fields_types: List[Byte_Type], buf=None):
        self._buffer = Buffer(buf)
        self._obj_fields_types = obj_fields_types
        self._field_number = len(self._obj_fields_types)
        assert self._obj_fields_types, 'The types of fields must be input'

    def __build_write_fmt(self, *obj_fields):
        fmt = ''
        t_sizes, t_values = [], []
        for t, v in zip(self._obj_fields_types, obj_fields):
            t_len = 0 if v is None else (len(v) if t.id > 10 else 1)
            fmt = fmt + (t.c * t_len)
            t_sizes.append(t_len)
            if t_len > 0: t_values.append(v) if t.id < 10 else t_values.extend(v)
        return fmt, t_sizes, t_values

    def write(self, *obj_fields):
        fmt, t_sizes, t_values = self.__build_write_fmt(*obj_fields)
        self._buffer.write_bytes(t_sizes)
        self._buffer.writes(fmt, *t_values)
        return self

    def write_list(self, obj_list):
        for obj in obj_list: self.write(*obj)
        return self

    def __build_read_fmt_len(self, v_lens):
        fmt = ''
        all_len = 0
        for t, l in zip(self._obj_fields_types, v_lens):
            fmt = fmt + t.c * l
            all_len = all_len + t.len * l
        return fmt, all_len

    def __build_read_obj(self, v_lens, v_list):
        start = 0
        for t, l in zip(self._obj_fields_types, v_lens):
            v = None
            if l > 0:
                end = start + l
                v = v_list[start] if l == 1 and t.id < 10 else list(v_list[start:end])
                start = end
            yield v

    def read_iter(self):
        self._buffer.seek()
        while True:
            v_lens = self._buffer.read_bytes(self._field_number)
            if not v_lens: break
            fmt, all_len = self.__build_read_fmt_len(v_lens)
            v_list = self._buffer.reads(fmt, all_len)
            if not v_list: break
            yield [v for v in self.__build_read_obj(v_lens, v_list)]

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self._buffer: self._buffer.close()



import os
from inspect import isfunction, ismethod

from box import Box
from dataclasses import dataclass

from ai_harness.tqdmutils import ProgressBar
from boltons.fileutils import mkdir_p
from pathlib import Path
import zipfile


def _instance_filter(handler):
    if handler is None: raise Exception('The Pipe handler can not be None')
    t = type(handler)
    if t == type: return handler()
    return handler


def list_file(path, pattern='*'):
    p = Path(path)
    if not p.exists(): print('Directory is not exists.')
    return [x.name for x in p.glob(pattern) if x.is_file()]


def list_dir(path, pattern='*'):
    p = Path(path)
    if not p.exists(): print('Directory is not exists.')
    return [x.name for x in p.glob(pattern) if x.is_dir()]


def list(path, pattern='*', file=True):
    return list_file(path, pattern) if file else list_dir(path, pattern)


def zip_files(out_file, *input_files):
    def addToZip(zf, path, zippath):
        if os.path.isfile(path):
            zf.write(path, zippath, zipfile.ZIP_DEFLATED)
        elif os.path.isdir(path):
            if zippath: zf.write(path, zippath)
            for nm in os.listdir(path):
                addToZip(zf, os.path.join(path, nm), os.path.join(zippath, nm))
        # else: ignore

    with zipfile.ZipFile(out_file, 'w') as zf:
        for path in input_files[2:]:
            zippath = os.path.basename(path)
            if not zippath: zippath = os.path.basename(os.path.dirname(path))
            if zippath in ('', os.curdir, os.pardir): zippath = ''
            addToZip(zf, path, zippath)


def join_path(*files):
    result = files[0]
    for file in files[1:]:
        if result is None:
            result = file
            continue
        if file is None: continue
        result = os.path.join(result, file)
    return result


def extract_zip(zip_file, dest_dir, members=None):
    if not zipfile.is_zipfile(zip_file): return
    mkdir_p(dest_dir)
    with zipfile.ZipFile(zip_file, 'r') as zf:
        zf.extractall(dest_dir, members)


class EmptyLineFilter():
    def __call__(self, line, *args):
        return line == '\n'


class FileLineReader():
    def __init__(self, bar_step_size=1000, exclude_empty_line=True):
        self._handlers = ()
        self._exclude_empty_line = exclude_empty_line
        self._bar = ProgressBar(bar_step_size)

    def pipe(self, *handlers):
        self._handlers = self._handlers + handlers
        return self

    def _read_line(self, file):
        with open(file, 'r') as f:
            count = 0
            while True:
                self._bar.update()
                read_line = f.readline()

                if len(read_line) == 0: return count

                if self._exclude_empty_line and read_line.strip() == '': continue

                input = read_line
                result = ()

                for handler in self._handlers:
                    if handler is None: continue

                    next = handler(input, result)

                    if not next and type(next) == bool:
                        break
                    elif next and type(next) != bool:
                        pass
                    else:
                        next = input

                    result = result + (input,)
                    input = next
                count = count + 1

    def read(self, file, *args):
        try:
            return self._read_line(file)
        finally:
            self._bar.close()


class PathNavigator():
    def on_item(self, onFolder=None, onFile=None, *args):
        pass

    def nav(self, dir, *args):
        pass


@dataclass
class NavState:
    folder_count = 0
    file_count = 0

    def folders(self, count):
        self.folder_count = self.folder_count + count
        return self

    def files(self, count):
        self.file_count = self.file_count + count
        return self

    def inc_folder(self):
        self.folder_count = self.folder_count + 1
        return self

    def inc_file(self):
        self.file_count = self.file_count + 1
        return self


class DirNavigator(PathNavigator):
    def __init__(self, folder_pattern=None, file_pattern=None, bar_step_size=10):
        self._folder_pattern = folder_pattern if folder_pattern else '*'
        self._file_pattern = file_pattern if file_pattern else '*'
        self._onFolder, self._onFile, self._onZipFile = None, None, None
        self._bar = ProgressBar(bar_step_size)
        self._folderFilters, self._fileFilters = (), ()

    def on_item(self, onFolder=None, onFile=None, onZipFile=None):
        self._onFolder, self._onFile, self._onZipFile = onFolder, onFile, onZipFile
        return self

    def filters(self, folderFilters=None, fileFilters=None):
        self._folderFilters, self._fileFilters = folderFilters, fileFilters
        return self

    def folder_filters(self, *folderFilters):
        self._folderFilters = self._folderFilters + folderFilters
        return self

    def file_filters(self, *fileFilters):
        self._fileFilters = self._fileFilters + fileFilters
        return self

    def __filter(self, filters, name, dir):
        if not filters: return True
        for filter in filters:
            if not filter(name, dir): return False
        return True

    def _loop_dir(self, parent, dir, state: NavState):
        file_dir = join_path(self._base_dir, parent, dir)
        relate_dir = join_path(parent, dir)
        folders = list_dir(file_dir, self._folder_pattern)
        files = list_file(file_dir, self._file_pattern)

        for d in folders:
            self._bar.update()
            if not self.__filter(self._folderFilters, d, relate_dir): continue
            state.inc_folder()
            if not self._onFolder or self._onFolder(d, relate_dir):
                self._loop_dir(relate_dir, d, state)

        for f in files:
            self._bar.update()
            if not self.__filter(self._fileFilters, f, relate_dir): continue
            if zipfile.is_zipfile(f) and self._onZipFile:
                self._onZipFile(f, relate_dir)
                continue
            state.inc_file()
            if self._onFile: self._onFile(f, relate_dir)

    def nav(self, dir, *args):
        navState = NavState()
        self._base_dir = dir
        self._loop_dir(None, '', navState)
        self._bar.close()
        return (navState.folder_count, navState.file_count)


class DirNavigatorWithZip(DirNavigator):
    def __init__(self, folder_pattern=None, file_pattern=None):
        super().__init__(folder_pattern, file_pattern)

    def __handle_zip_file(self, unzip_dir):
        def handle_zip(zipfile, dir):
            to_zip_dir = join_path(unzip_dir, dir, 'unzip_' + os.path.basename(zipfile))
            extract_zip(zipfile, to_zip_dir)
            super().nav(to_zip_dir)

        return handle_zip

    def nav(self, dir, unzip_dir=None, *args):
        super().on_item(onZipFile=self.__handle_zip_file(unzip_dir))
        super().nav(dir)


class To_Json():
    def __call__(self, line, *args):
        return Box.from_json(line)


class Line_origin():
    def __call__(self, input, previous_input: tuple):
        return previous_input[0]


class Line_Writer():
    def __init__(self, output_file):
        self._out_writer = open(self._get_filename(output_file), 'w')

    def _get_filename(self, output_file):
        if os.path.exists(output_file):
            suffix = output_file.split('.')
            if suffix and suffix[-1].isnumeric():
                return "".join(suffix[:-1]) + '.' + str(int(suffix) + 1)
            return output_file + '.0'
        return output_file

    def __call__(self, input, previous_input: tuple = None):
        self._out_writer.write(input)

    def close(self):
        if not self._out_writer.closed:
            self._out_writer.close()


class JsonFileLineReader(FileLineReader):
    def __init__(self, bar_step_size=1000):
        super().__init__(bar_step_size)
        self.handlers = [To_Json]


class FileLineReadAndWrite():
    def __init__(self, out_dir, bar_step_size=100):
        self.bar_step_size = bar_step_size
        self.out_dir = out_dir
        self.filters = ()

    def filter(self, *filters):
        self.filters = self.filters + filters

    def read(self, file, relate_dir, writer=None):
        myWriter = writer
        if not writer: myWriter = self._writer(file, relate_dir, self.out_dir)
        filters = self.filters + (myWriter,)
        file_reader = FileLineReader(self.bar_step_size).pipe(*filters)
        file_reader.read(file)
        if not writer: myWriter.close()

    def _writer(self, file, relate_dir, out_dir):
        out_dir = join_path(out_dir, relate_dir)
        mkdir_p(out_dir)
        out_file = join_path(out_dir, os.path.basename(file))
        return Line_Writer(out_file)


class DefaultDirectoryFilter():
    def __init__(self, output, unzip_dir=None, folder_pattern='*', file_pattern='*', bar_step_size=100):
        self.browser = self._create_browser(unzip_dir, folder_pattern, file_pattern)
        self.readWrite = FileLineReadAndWrite(output, bar_step_size=bar_step_size)

    def _create_browser(self, unzip_dir, folder_pattern='*', file_pattern='*'):
        if unzip_dir:
            return DirNavigatorWithZip(folder_pattern, file_pattern)
        else:
            return DirNavigator(folder_pattern, file_pattern)

    def line_filters(self, *filters):
        self.readWrite.filter(*filters)
        return self

    def path_filters(self, folder_filters, file_filters):
        self.browser.filters(folder_filters, file_filters)
        return self

    def _on_file(self, writer=None):
        def onFile(file, relate_dir):
            file = join_path(self._base_dir, relate_dir, file)
            self.readWrite.read(file, relate_dir, writer)

        return onFile

    def handle(self, dir, outfile=None):
        writer = None
        self._base_dir = dir
        if outfile: writer = Line_Writer(outfile)
        self.browser.on_item(onFile=self._on_file(writer)).nav(dir)
        if writer: writer.close()


class DefaultJsonDirectoryFilter(DefaultDirectoryFilter):
    def __init__(self, output, unzip_dir=None, dir_pattern='*', file_pattern='*', bar_step_size=100):
        super().__init__(output, unzip_dir, dir_pattern, file_pattern, bar_step_size)
        super().line_filters(To_Json())



import json
import re
import sys
import copy
import types
import inspect
import keyword

__all__ = ['configclass',
           'field',
           'Field',
           'FrozenInstanceError',
           'InitVar',
           'MISSING',

           # Helper functions.
           'fields',
           'asdict',
           'astuple',
           'make_configclass',
           'replace',
           'is_configclass',
           ]


# Raised when an attempt is made to modify a frozen class.
class FrozenInstanceError(AttributeError): pass


class _HAS_DEFAULT_FACTORY_CLASS:
    def __repr__(self):
        return '<factory>'


_HAS_DEFAULT_FACTORY = _HAS_DEFAULT_FACTORY_CLASS()


# A sentinel object to detect if a parameter is supplied or not.  Use
# a class to give it a better repr.
class _MISSING_TYPE:
    pass


MISSING = _MISSING_TYPE()

# Since most per-field metadata will be unused, create an empty
# read-only proxy that can be shared among all fields.
_EMPTY_METADATA = types.MappingProxyType({})


# Markers for the various kinds of fields and pseudo-fields.
class _FIELD_BASE:
    def __init__(self, name):
        self.name = name

    def __repr__(self):
        return self.name


_FIELD = _FIELD_BASE('_FIELD')
_FIELD_CLASSVAR = _FIELD_BASE('_FIELD_CLASSVAR')
_FIELD_INITVAR = _FIELD_BASE('_FIELD_INITVAR')

# The name of an attribute on the class where we store the Field
# objects.  Also used to check if a class is a Data Class.
_FIELDS = '__configclass_fields__'

# The name of an attribute on the class that stores the parameters to
# @configclass.
_PARAMS = '__configclass_params__'

# The name of the function, that if it exists, is called at the end of
# __init__.
_POST_INIT_NAME = '__post_init__'

# String regex that string annotations for ClassVar or InitVar must match.
# Allows "identifier.identifier[" or "identifier[".
# https://bugs.python.org/issue33453 for details.
_MODULE_IDENTIFIER_RE = re.compile(r'^(?:\s*(\w+)\s*\.)?\s*(\w+)')


class _InitVarMeta(type):
    def __getitem__(self, params):
        return self


class InitVar(metaclass=_InitVarMeta):
    pass


# Instances of Field are only ever created from within this module,
# and only from the field() function, although Field instances are
# exposed externally as (conceptually) read-only objects.
#
# name and type are filled in after the fact, not in __init__.
# They're not known at the time this class is instantiated, but it's
# convenient if they're available later.
#
# When cls._FIELDS is filled in with a list of Field objects, the name
# and type fields will have been populated.
class Field:
    __slots__ = ('name',
                 'type',
                 'default',
                 'help',
                 'default_factory',
                 'repr',
                 'hash',
                 'init',
                 'compare',
                 'metadata',
                 '_field_type',  # Private: not to be used by user code.
                 )

    def __init__(self, default, default_factory, help, init, repr, hash, compare,
                 metadata):
        self.name = None
        self.type = None
        self.default = default
        self.default_factory = default_factory
        self.help = help
        self.init = init
        self.repr = repr
        self.hash = hash
        self.compare = compare
        self.metadata = (_EMPTY_METADATA
                         if metadata is None or len(metadata) == 0 else
                         types.MappingProxyType(metadata))
        self._field_type = None

    def __repr__(self):
        return ('Field('
                f'name={self.name!r},'
                f'type={self.type!r},'
                f'default={self.default!r},'
                f'default_factory={self.default_factory!r},'
                f'help={self.help!r},'
                f'init={self.init!r},'
                f'repr={self.repr!r},'
                f'hash={self.hash!r},'
                f'compare={self.compare!r},'
                f'metadata={self.metadata!r},'
                f'_field_type={self._field_type}'
                ')')

    def __set_name__(self, owner, name):
        func = getattr(type(self.default), '__set_name__', None)
        if func:
            # There is a __set_name__ method on the descriptor, call
            # it.
            func(self.default, owner, name)


class _ConfigclassParams:
    __slots__ = ('init',
                 'repr',
                 'eq',
                 'order',
                 'unsafe_hash',
                 'frozen',
                 )

    def __init__(self, init, repr, eq, order, unsafe_hash, frozen):
        self.init = init
        self.repr = repr
        self.eq = eq
        self.order = order
        self.unsafe_hash = unsafe_hash
        self.frozen = frozen

    def __repr__(self):
        return ('_ConfigclassParams('
                f'init={self.init!r},'
                f'repr={self.repr!r},'
                f'eq={self.eq!r},'
                f'order={self.order!r},'
                f'unsafe_hash={self.unsafe_hash!r},'
                f'frozen={self.frozen!r}'
                ')')


# This function is used instead of exposing Field creation directly,
# so that a type checker can be told (via overloads) that this is a
# function whose type depends on its parameters.
def field(default=MISSING, help='', default_factory=MISSING, init=True, repr=True,
          hash=None, compare=True, metadata=None):
    if default is not MISSING and default_factory is not MISSING:
        raise ValueError('cannot specify both default and default_factory')
    return Field(default, default_factory, help, init, repr, hash, compare,
                 metadata)


def _tuple_str(obj_name, fields):
    # Return a string representing each field of obj_name as a tuple
    # member.  So, if fields is ['x', 'y'] and obj_name is "self",
    # return "(self.x,self.y)".

    # Special case for the 0-tuple.
    if not fields:
        return '()'
    # Note the trailing comma, needed if this turns out to be a 1-tuple.
    return f'({",".join([f"{obj_name}.{f.name}" for f in fields])},)'


def _create_fn(name, args, body, *, globals=None, locals=None,
               return_type=MISSING):
    # Note that we mutate locals when exec() is called.  Caller
    # beware!  The only callers are internal to this module, so no
    # worries about external callers.
    if locals is None:
        locals = {}
    return_annotation = ''
    if return_type is not MISSING:
        locals['_return_type'] = return_type
        return_annotation = '->_return_type'
    args = ','.join(args)
    body = '\n'.join(f' {b}' for b in body)

    # Compute the text of the entire function.
    txt = f'def {name}({args}){return_annotation}:\n{body}'

    exec(txt, globals, locals)
    return locals[name]


def _field_assign(frozen, name, value, self_name):
    if frozen:
        return f'object.__setattr__({self_name},{name!r},{value})'
    return f'{self_name}.{name}={value}'


def _field_init(f, frozen, globals, self_name):
    # Return the text of the line in the body of __init__ that will
    # initialize this field.

    default_name = f'_dflt_{f.name}'
    if f.default_factory is not MISSING:
        if f.init:
            # This field has a default factory.  If a parameter is
            # given, use it.  If not, call the factory.
            globals[default_name] = f.default_factory
            value = (f'{default_name}() '
                     f'if {f.name} is _HAS_DEFAULT_FACTORY '
                     f'else {f.name}')
        else:
            globals[default_name] = f.default_factory
            value = f'{default_name}()'
    else:
        # No default factory.
        if f.init:
            if f.default is MISSING:
                # There's no default, just do an assignment.
                value = f.name
            elif f.default is not MISSING:
                globals[default_name] = f.default
                value = f.name
        else:
            # This field does not need initialization.  Signify that
            # to the caller by returning None.
            return None

    # Only test this now, so that we can create variables for the
    # default.  However, return None to signify that we're not going
    # to actually do the assignment statement for InitVars.
    if f._field_type == _FIELD_INITVAR:
        return None

    # Now, actually generate the field assignment.
    return _field_assign(frozen, f.name, value, self_name)


def _init_param(f):
    # Return the __init__ parameter string for this field.  For
    # example, the equivalent of 'x:int=3' (except instead of 'int',
    # reference a variable set to int, and instead of '3', reference a
    # variable set to 3).
    if f.default is MISSING and f.default_factory is MISSING:
        # There's no default, and no default_factory, just output the
        # variable name and type.
        default = ''
    elif f.default is not MISSING:
        # There's a default, this will be the name that's used to look
        # it up.
        default = f'=_dflt_{f.name}'
    elif f.default_factory is not MISSING:
        # There's a factory function.  Set a marker.
        default = '=_HAS_DEFAULT_FACTORY'
    return f'{f.name}:_type_{f.name}{default}'


def _init_fn(fields, frozen, has_post_init, self_name):
    seen_default = False
    for f in fields:
        # Only consider fields in the __init__ call.
        if f.init:
            if not (f.default is MISSING and f.default_factory is MISSING):
                seen_default = True
            elif seen_default:
                raise TypeError(f'non-default argument {f.name!r} '
                                'follows default argument')

    globals = {'MISSING': MISSING,
               '_HAS_DEFAULT_FACTORY': _HAS_DEFAULT_FACTORY}

    body_lines = []
    for f in fields:
        line = _field_init(f, frozen, globals, self_name)
        # line is None means that this field doesn't require
        # initialization (it's a pseudo-field).  Just skip it.
        if line:
            body_lines.append(line)

    # Does this class have a post-init function?
    if has_post_init:
        params_str = ','.join(f.name for f in fields
                              if f._field_type is _FIELD_INITVAR)
        body_lines.append(f'{self_name}.{_POST_INIT_NAME}({params_str})')

    # If no body lines, use 'pass'.
    if not body_lines:
        body_lines = ['pass']

    locals = {f'_type_{f.name}': f.type for f in fields}
    return _create_fn('__init__',
                      [self_name] + [_init_param(f) for f in fields if f.init],
                      body_lines,
                      locals=locals,
                      globals=globals,
                      return_type=None)


def _repr_fn(fields):
    return _create_fn('__repr__',
                      ('self',),
                      ['return self.__class__.__qualname__ + f"(' +
                       ', '.join([f"{f.name}={{self.{f.name}!r}}"
                                  for f in fields]) +
                       ')"'])


def _frozen_get_del_attr(cls, fields):
    # XXX: globals is modified on the first call to _create_fn, then
    # the modified version is used in the second call.  Is this okay?
    globals = {'cls': cls,
               'FrozenInstanceError': FrozenInstanceError}
    if fields:
        fields_str = '(' + ','.join(repr(f.name) for f in fields) + ',)'
    else:
        # Special case for the zero-length tuple.
        fields_str = '()'
    return (_create_fn('__setattr__',
                       ('self', 'name', 'value'),
                       (f'if type(self) is cls or name in {fields_str}:',
                        ' raise FrozenInstanceError(f"cannot assign to field {name!r}")',
                        f'super(cls, self).__setattr__(name, value)'),
                       globals=globals),
            _create_fn('__delattr__',
                       ('self', 'name'),
                       (f'if type(self) is cls or name in {fields_str}:',
                        ' raise FrozenInstanceError(f"cannot delete field {name!r}")',
                        f'super(cls, self).__delattr__(name)'),
                       globals=globals),
            )


def _cmp_fn(name, op, self_tuple, other_tuple):
    # Create a comparison function.  If the fields in the object are
    # named 'x' and 'y', then self_tuple is the string
    # '(self.x,self.y)' and other_tuple is the string
    # '(other.x,other.y)'.

    return _create_fn(name,
                      ('self', 'other'),
                      ['if other.__class__ is self.__class__:',
                       f' return {self_tuple}{op}{other_tuple}',
                       'return NotImplemented'])


def _hash_fn(fields):
    self_tuple = _tuple_str('self', fields)
    return _create_fn('__hash__',
                      ('self',),
                      [f'return hash({self_tuple})'])


def _is_classvar(a_type, typing):
    # This test uses a typing internal class, but it's the best way to
    # test if this is a ClassVar.
    return type(a_type) is typing.ClassVar


def _is_initvar(a_type, configclasses):
    # The module we're checking against is the module we're
    # currently in (configclasses.py).
    return a_type is configclasses.InitVar


def _is_type(annotation, cls, a_module, a_type, is_type_predicate):
    match = _MODULE_IDENTIFIER_RE.match(annotation)
    if match:
        ns = None
        module_name = match.group(1)
        if not module_name:
            # No module name, assume the class's module did
            # "from configclasses import InitVar".
            ns = sys.modules.get(cls.__module__).__dict__
        else:
            # Look up module_name in the class's module.
            module = sys.modules.get(cls.__module__)
            if module and module.__dict__.get(module_name) is a_module:
                ns = sys.modules.get(a_type.__module__).__dict__
        if ns and is_type_predicate(ns.get(match.group(2)), a_module):
            return True
    return False


def _get_field(cls, a_name, a_type):
    default = getattr(cls, a_name, MISSING)
    if isinstance(default, Field):
        f = default
    else:
        if isinstance(default, types.MemberDescriptorType):
            # This is a field in __slots__, so it has no default value.
            default = MISSING
        f = field(default=default)

    # Only at this point do we know the name and the type.  Set them.
    f.name = a_name
    f.type = a_type

    # Assume it's a normal field until proven otherwise.  We're next
    # going to decide if it's a ClassVar or InitVar, everything else
    # is just a normal field.
    f._field_type = _FIELD

    typing = sys.modules.get('typing')
    if typing:
        if (_is_classvar(a_type, typing)
                or (isinstance(f.type, str)
                    and _is_type(f.type, cls, typing, typing.ClassVar,
                                 _is_classvar))):
            f._field_type = _FIELD_CLASSVAR

    # If the type is InitVar, or if it's a matching string annotation,
    # then it's an InitVar.
    if f._field_type is _FIELD:
        # The module we're checking against is the module we're
        # currently in (configclasses.py).
        configclasses = sys.modules[__name__]
        if (_is_initvar(a_type, configclasses)
                or (isinstance(f.type, str)
                    and _is_type(f.type, cls, configclasses, configclasses.InitVar,
                                 _is_initvar))):
            f._field_type = _FIELD_INITVAR

    # Special restrictions for ClassVar and InitVar.
    if f._field_type in (_FIELD_CLASSVAR, _FIELD_INITVAR):
        if f.default_factory is not MISSING:
            raise TypeError(f'field {f.name} cannot have a '
                            'default factory')

    # For real fields, disallow mutable defaults for known types.
    if f._field_type is _FIELD and isinstance(f.default, (list, dict, set)):
        raise ValueError(f'mutable default {type(f.default)} for field '
                         f'{f.name} is not allowed: use default_factory')

    return f


def _set_new_attribute(cls, name, value):
    # Never overwrites an existing attribute.  Returns True if the
    # attribute already exists.
    if name in cls.__dict__:
        return True
    setattr(cls, name, value)
    return False


def _hash_set_none(cls, fields):
    return None


def _hash_add(cls, fields):
    flds = [f for f in fields if (f.compare if f.hash is None else f.hash)]
    return _hash_fn(flds)


def _hash_exception(cls, fields):
    # Raise an exception.
    raise TypeError(f'Cannot overwrite attribute __hash__ '
                    f'in class {cls.__name__}')


#
#                +-------------------------------------- unsafe_hash?
#                |      +------------------------------- eq?
#                |      |      +------------------------ frozen?
#                |      |      |      +----------------  has-explicit-hash?
#                |      |      |      |
#                |      |      |      |        +-------  action
#                |      |      |      |        |
#                v      v      v      v        v
_hash_action = {(False, False, False, False): None,
                (False, False, False, True): None,
                (False, False, True, False): None,
                (False, False, True, True): None,
                (False, True, False, False): _hash_set_none,
                (False, True, False, True): None,
                (False, True, True, False): _hash_add,
                (False, True, True, True): None,
                (True, False, False, False): _hash_add,
                (True, False, False, True): _hash_exception,
                (True, False, True, False): _hash_add,
                (True, False, True, True): _hash_exception,
                (True, True, False, False): _hash_add,
                (True, True, False, True): _hash_exception,
                (True, True, True, False): _hash_add,
                (True, True, True, True): _hash_exception,
                }


def _process_class(cls, init, repr, eq, order, unsafe_hash, frozen):
    fields = {}

    setattr(cls, _PARAMS, _ConfigclassParams(init, repr, eq, order,
                                             unsafe_hash, frozen))

    any_frozen_base = False
    has_configclass_bases = False
    for b in cls.__mro__[-1:0:-1]:
        # Only process classes that have been processed by our
        # decorator.  That is, they have a _FIELDS attribute.
        base_fields = getattr(b, _FIELDS, None)
        if base_fields:
            has_configclass_bases = True
            for f in base_fields.values():
                fields[f.name] = f
            if getattr(b, _PARAMS).frozen:
                any_frozen_base = True

    cls_annotations = cls.__dict__.get('__annotations__', {})

    # Now find fields in our class.  While doing so, validate some
    # things, and set the default values (as class attributes) where
    # we can.
    cls_fields = [_get_field(cls, name, type)
                  for name, type in cls_annotations.items()]
    for f in cls_fields:
        fields[f.name] = f

        # If the class attribute (which is the default value for this
        # field) exists and is of type 'Field', replace it with the
        # real default.  This is so that normal class introspection
        # sees a real default value, not a Field.
        if isinstance(getattr(cls, f.name, None), Field):
            if f.default is MISSING:
                delattr(cls, f.name)
            else:
                setattr(cls, f.name, f.default)

    # Do we have any Field members that don't also have annotations?
    for name, value in cls.__dict__.items():
        if isinstance(value, Field) and not name in cls_annotations:
            raise TypeError(f'{name!r} is a field but has no type annotation')

    # Check rules that apply if we are derived from any configclasses.
    if has_configclass_bases:
        # Raise an exception if any of our bases are frozen, but we're not.
        if any_frozen_base and not frozen:
            raise TypeError('cannot inherit non-frozen configclass from a '
                            'frozen one')

        # Raise an exception if we're frozen, but none of our bases are.
        if not any_frozen_base and frozen:
            raise TypeError('cannot inherit frozen configclass from a '
                            'non-frozen one')

    setattr(cls, _FIELDS, fields)

    # Was this class defined with an explicit __hash__?  Note that if
    # __eq__ is defined in this class, then python will automatically
    # set __hash__ to None.  This is a heuristic, as it's possible
    # that such a __hash__ == None was not auto-generated, but it
    # close enough.
    class_hash = cls.__dict__.get('__hash__', MISSING)
    has_explicit_hash = not (class_hash is MISSING or
                             (class_hash is None and '__eq__' in cls.__dict__))

    # If we're generating ordering methods, we must be generating the
    # eq methods.
    if order and not eq:
        raise ValueError('eq must be true if order is true')

    if init:
        # Does this class have a post-init function?
        has_post_init = hasattr(cls, _POST_INIT_NAME)

        # Include InitVars and regular fields (so, not ClassVars).
        flds = [f for f in fields.values()
                if f._field_type in (_FIELD, _FIELD_INITVAR)]
        _set_new_attribute(cls, '__init__',
                           _init_fn(flds,
                                    frozen,
                                    has_post_init,
                                    # The name to use for the "self"
                                    # param in __init__.  Use "self"
                                    # if possible.
                                    '__configclass_self__' if 'self' in fields
                                    else 'self',
                                    ))

    # Get the fields as a list, and include only real fields.  This is
    # used in all of the following methods.
    field_list = [f for f in fields.values() if f._field_type is _FIELD]

    if repr:
        flds = [f for f in field_list if f.repr]
        _set_new_attribute(cls, '__repr__', _repr_fn(flds))

    if eq:
        # Create _eq__ method.  There's no need for a __ne__ method,
        # since python will call __eq__ and negate it.
        flds = [f for f in field_list if f.compare]
        self_tuple = _tuple_str('self', flds)
        other_tuple = _tuple_str('other', flds)
        _set_new_attribute(cls, '__eq__',
                           _cmp_fn('__eq__', '==',
                                   self_tuple, other_tuple))

    if order:
        # Create and set the ordering methods.
        flds = [f for f in field_list if f.compare]
        self_tuple = _tuple_str('self', flds)
        other_tuple = _tuple_str('other', flds)
        for name, op in [('__lt__', '<'),
                         ('__le__', '<='),
                         ('__gt__', '>'),
                         ('__ge__', '>='),
                         ]:
            if _set_new_attribute(cls, name,
                                  _cmp_fn(name, op, self_tuple, other_tuple)):
                raise TypeError(f'Cannot overwrite attribute {name} '
                                f'in class {cls.__name__}. Consider using '
                                'functools.total_ordering')

    if frozen:
        for fn in _frozen_get_del_attr(cls, field_list):
            if _set_new_attribute(cls, fn.__name__, fn):
                raise TypeError(f'Cannot overwrite attribute {fn.__name__} '
                                f'in class {cls.__name__}')

    # Decide if/how we're going to create a hash function.
    hash_action = _hash_action[bool(unsafe_hash),
                               bool(eq),
                               bool(frozen),
                               has_explicit_hash]
    if hash_action:
        # No need to call _set_new_attribute here, since by the time
        # we're here the overwriting is unconditional.
        cls.__hash__ = hash_action(cls, field_list)

    if not getattr(cls, '__doc__'):
        # Create a class doc-string.
        cls.__doc__ = (cls.__name__ +
                       str(inspect.signature(cls)).replace(' -> None', ''))

    return cls


def configclass(_cls=None, *, init=True, repr=True, eq=True, order=False,
                unsafe_hash=False, frozen=False):
    def wrap(cls):
        return _process_class(cls, init, repr, eq, order, unsafe_hash, frozen)

    # See if we're being called as @configclass or @configclass().
    if _cls is None:
        # We're called with parens.
        return wrap

    return wrap(_cls)


def fields(class_or_instance):
    # Might it be worth caching this, per class?
    try:
        fields = getattr(class_or_instance, _FIELDS)
    except AttributeError:
        raise TypeError('must be called with a configclass type or instance')

    # Exclude pseudo-fields.  Note that fields is sorted by insertion
    # order, so the order of the tuple is as the fields were defined.
    return tuple(f for f in fields.values() if f._field_type is _FIELD)


def _is_configclass_instance(obj):
    return not isinstance(obj, type) and hasattr(obj, _FIELDS)


def is_configclass(obj):
    return hasattr(obj, _FIELDS)


def asdict(obj, *, dict_factory=dict):
    if not _is_configclass_instance(obj):
        raise TypeError("asdict() should be called on configclass instances")
    return _asdict_inner(obj, dict_factory)


def _asdict_inner(obj, dict_factory):
    if _is_configclass_instance(obj):
        result = []
        for f in fields(obj):
            value = _asdict_inner(getattr(obj, f.name), dict_factory)
            result.append((f.name, value))
        return dict_factory(result)
    elif isinstance(obj, (list, tuple)):
        return type(obj)(_asdict_inner(v, dict_factory) for v in obj)
    elif isinstance(obj, dict):
        return type(obj)((_asdict_inner(k, dict_factory), _asdict_inner(v, dict_factory))
                         for k, v in obj.items())
    else:
        return copy.deepcopy(obj)


def astuple(obj, *, tuple_factory=tuple):
    if not _is_configclass_instance(obj):
        raise TypeError("astuple() should be called on configclass instances")
    return _astuple_inner(obj, tuple_factory)


def _astuple_inner(obj, tuple_factory):
    if _is_configclass_instance(obj):
        result = []
        for f in fields(obj):
            value = _astuple_inner(getattr(obj, f.name), tuple_factory)
            result.append(value)
        return tuple_factory(result)
    elif isinstance(obj, (list, tuple)):
        return type(obj)(_astuple_inner(v, tuple_factory) for v in obj)
    elif isinstance(obj, dict):
        return type(obj)((_astuple_inner(k, tuple_factory), _astuple_inner(v, tuple_factory))
                         for k, v in obj.items())
    else:
        return copy.deepcopy(obj)


def make_configclass(cls_name, fields, *, bases=(), namespace=None, init=True,
                     repr=True, eq=True, order=False, unsafe_hash=False,
                     frozen=False):
    if namespace is None:
        namespace = {}
    else:
        # Copy namespace since we're going to mutate it.
        namespace = namespace.copy()

    # While we're looking through the field names, validate that they
    # are identifiers, are not keywords, and not duplicates.
    seen = set()
    anns = {}
    for item in fields:
        if isinstance(item, str):
            name = item
            tp = 'typing.Any'
        elif len(item) == 2:
            name, tp, = item
        elif len(item) == 3:
            name, tp, spec = item
            namespace[name] = spec
        else:
            raise TypeError(f'Invalid field: {item!r}')

        if not isinstance(name, str) or not name.isidentifier():
            raise TypeError(f'Field names must be valid identifers: {name!r}')
        if keyword.iskeyword(name):
            raise TypeError(f'Field names must not be keywords: {name!r}')
        if name in seen:
            raise TypeError(f'Field name duplicated: {name!r}')

        seen.add(name)
        anns[name] = tp

    namespace['__annotations__'] = anns

    cls = types.new_class(cls_name, bases, {}, lambda ns: ns.update(namespace))
    return configclass(cls, init=init, repr=repr, eq=eq, order=order,
                       unsafe_hash=unsafe_hash, frozen=frozen)


def merge_fields(dest_cls, *src_classes):
    for src_cls in src_classes:
        src_fields = getattr(src_cls, _FIELDS, None)
        dest_fields = getattr(dest_cls, _FIELDS, {})

        for f in src_fields.values():
            dest_fields[f.name] = f

            setattr(dest_cls, f.name, f.default)

        # setattr(dest_cls, _FIELDS, dest_fields)

    return dest_cls


def export(from_obj, to_obj_or_class):
    to_fields = fields(to_obj_or_class)
    to_obj = to_obj_or_class() if type(to_obj_or_class) == type else to_obj_or_class
    for f in to_fields:
        if hasattr(from_obj, f.name):
            setattr(to_obj, f.name, getattr(from_obj, f.name))
    return to_obj


def replace(obj, **changes):
    if not _is_configclass_instance(obj):
        raise TypeError("replace() should be called on configclass instances")

    # It's an error to have init=False fields in 'changes'.
    # If a field is not in 'changes', read its value from the provided obj.

    for f in getattr(obj, _FIELDS).values():
        if not f.init:
            # Error if this field is specified in changes.
            if f.name in changes:
                raise ValueError(f'field {f.name} is declared with '
                                 'init=False, it cannot be specified with '
                                 'replace()')
            continue

        if f.name not in changes:
            changes[f.name] = getattr(obj, f.name)

    # Create the new object, which calls __init__() and
    # __post_init__() (if defined), using all of the init fields we've
    # added and/or left in 'changes'.  If there are values supplied in
    # changes that aren't fields, this will correctly raise a
    # TypeError.
    return obj.__class__(**changes)


def to_json_string(obj):
    """
    Serializes this instance to a JSON string.
    """
    return json.dumps(asdict(obj), indent=2)



from ai_harness import harnessutils as utils
from ai_harness.inspector import Inspector
import argparse
from ai_harness.configclasses import configclass, field, fields, Field, is_configclass, make_configclass, merge_fields, \
    export, to_json_string


def arg(value, help):
    return field(default=value, metadata={"help": help})


class ConfigInspector:
    def __init__(self, config):
        self.config = config
        self.fields = self._get_fields(config)
        self.configClasses = self._get_configClasses(config)

    def _get_fields(self, config):
        fieldDict = dict()
        for field in fields(config): fieldDict.setdefault(field.name, field)
        return fieldDict

    def _get_configClasses(self, config):
        configClassesDict = dict()
        for k, v in config.__dict__.items():
            if v is None: continue
            if is_configclass(v): configClassesDict.setdefault(k, v)
        return configClassesDict

    def is_configClass(self, fieldName):
        return self.configClasses.__contains__(fieldName)

    def get_field(self, name):
        return self.fields.get(name)

    def get_field_with_parent(self, name, root_parent=None):
        root_parent = self.config if not root_parent else root_parent
        this_field = self._get_fields(root_parent).get(name)
        if this_field: return root_parent, this_field
        for sub_name, sub_config in self._get_configClasses(root_parent).items():
            parent, this_field = self.get_field_with_parent(name, sub_config)
            if this_field: return sub_config, this_field
        return None, None

    def set(self, name, value=None, help=None, parse_name=False):
        parent = self.config
        if parse_name:
            attributes = name.split('.')
            name = attributes[-1]

            for attr in attributes[:-1]:
                parent = self._get_configClasses(parent).get(attr)
            this_field = self._get_fields(parent).get(name)
        else:
            parent, this_field = self.get_field_with_parent(name, parent)

        if this_field is None: return
        if value is not None: setattr(parent, name, this_field.type(value))
        if help is not None and help != '': this_field.help = help

    def value(self, name):
        getattr(self.config, name)

    def help(self, name):
        field = self.get_field(name)
        if field is not None: return field.help
        if hasattr(self.config, name):
            attr = getattr(self.config, name)
            if attr is not None and hasattr(attr, 'help'): return getattr(attr, 'help')
        return None


class XmlConfiguration:
    def __init__(self, config):
        if config is None: raise ValueError("target config type can not be none.")

        if type(config) == type: self.config = config()
        self.configInspector = ConfigInspector(self.config)

    def __set_xml2arg(self, groupInspector, argXml):
        argName = argXml['name'].replace('-', '_')
        groupInspector.set(argName, argXml['default'], argXml['help'])

    def __set_xml2group(self, groupObj, groupXml):
        if not hasattr(groupXml, 'arg'): return
        groupInspector = ConfigInspector(groupObj)
        if isinstance(groupXml.arg, list):
            for arg in groupXml.arg: self.__set_xml2arg(groupInspector, arg)
        else:
            self.__set_xml2arg(groupInspector, groupXml.arg)

    def __find_set_xml2group(self, config, groupXml):
        groupName = groupXml['name'].replace('-', '_')
        groupHelp = groupXml['help']
        groupField = self.configInspector.get_field(groupName)
        groupObj = getattr(config, groupName)
        if groupField is not None:
            self.configInspector.set(groupName, help=groupHelp)
        else:
            if groupObj is None: return
            setattr(groupObj, 'help', groupHelp)
        self.__set_xml2group(groupObj, groupXml)

    def load(self, xml_files: []):
        """
        Function for loading the xml_file into the configuration object.
        This function can be called multiple times to load multiple xml files.
        And the configuration value will be overrode by the following xml configuration.
        :param xml_file:
        :return: configuration object
        """
        if xml_files is None: return self.config
        for xml_file in xml_files:
            xml = utils.load_xml(xml_file)

            if xml is None: return self.config

            # if has group, set the args in the groups
            if hasattr(xml.configuration, 'group'):
                if isinstance(xml.configuration.group, list):
                    for group in xml.configuration.group: self.__find_set_xml2group(self.config, group)
                else:
                    self.__find_set_xml2group(self.config, xml.configuration.group)
            ## set other args
            if hasattr(xml.configuration, 'arg'): self.__set_xml2group(self.config, xml.configuration)

        return self.config


class ComplexArguments:
    def __init__(self, sub_arg_objs: dict, with_group_prefix=False, conflict_error=False):
        self._conflict_handler = 'error' if conflict_error else 'resolve'
        self._parser = argparse.ArgumentParser(conflict_handler=self._conflict_handler)
        self._subparsers = self._parser.add_subparsers(help='sub-command help', dest='cmd')
        self._subparsers.required = True
        self._sub_arg_objs = sub_arg_objs
        self._with_group_prefix = with_group_prefix
        self._conflict_error = conflict_error
        self._arg_objs = {}
        self.__create_args()

    def __create_args(self):
        for sub, arg_obj in self._sub_arg_objs.items():
            if arg_obj is None: continue
            parser = self._subparsers.add_parser(sub, conflict_handler=self._conflict_handler,
                                                 help='{} help'.format(sub))
            self._arg_objs[sub] = Arguments(arg_obj, parser, self._with_group_prefix, self._conflict_error)

    def _get_arg_obj(self, sub, args):
        argument: Arguments = self._arg_objs[sub]
        return argument.set_values(args, self._with_group_prefix)

    def parse(self, args=None):
        args, _ = self._parser.parse_known_args(args)
        # print("parsed input args:{}".format(str(args)))
        if not self._arg_objs: return None, None
        return args.cmd, self._get_arg_obj(args.cmd, args)


class Arguments:
    def __init__(self, configObj, parser=None, with_group_prefix=False, conflict_error=False):
        self.parser = argparse.ArgumentParser(
            conflict_handler='error' if conflict_error else 'resolve') if parser is None else parser
        self.destObj = configObj
        self.with_group_prefix = with_group_prefix
        self.groups = dict()
        self.configInspector = ConfigInspector(self.destObj)
        self._arg_obj(self.configInspector, self.parser)

    def __get_type_action(self, field):
        action = 'store'
        if field.type == bool:
            if field.default:
                return 'store_false'
            else:
                return 'store_true'
        return action

    def __get_group(self, groupName, help=''):
        group = self.groups.get(groupName)
        if group is not None: return group
        group = self.parser.add_argument_group(groupName, help)
        self.groups.setdefault(groupName, group)
        return group

    def _arg(self, field, v, parser, group=None):
        name = field.name
        if group is not None: name = group + '.' + name

        action = self.__get_type_action(field)
        required = True
        if v is None: required = False
        name = name.replace('_', '-')

        parser.add_argument('--' + name,
                            default=v,
                            required=required,
                            action=action,
                            help=field.help)
        return self

    def _arg_obj(self, configInspector, parser, groupName=None):
        for name, field in configInspector.fields.items():
            v = configInspector.value(name)
            if not configInspector.is_configClass(name):
                self._arg(field, v, parser, groupName)

        for k, v in configInspector.configClasses.items():
            parser = self.__get_group(k, configInspector.help(k))
            group_name = k if self.with_group_prefix else None
            self._arg_obj(ConfigInspector(v), parser, group_name)

        return self

    def set_values(self, args, with_group_prefix=False):
        for k, v in args.__dict__.items():
            if v: self.configInspector.set(k, v, None, with_group_prefix)
        return self.destObj

    def parse(self, args=None, with_group_prefix=False):
        args, _ = self.parser.parse_known_args(args)

        return self.set_values(args, with_group_prefix)



import codecs

import chardet


def str_q2b(s):
    res = ""
    for u in s:
        c = ord(u)
        if c == 12288:
            c = 32
        elif 65281 <= c <= 65374:
            c -= 65248
        res += chr(c)
    return res


def check_file_encoding(file_name):
    f = open(file_name, 'rb')
    data = f.read()
    detect_result = chardet.detect(data)
    print(detect_result)
    return detect_result


def generate_single_pairs_from_multi_turn(utterances):
    pairs = []
    for index in range(len(utterances) - 1):
        pairs.append((utterances[index], utterances[index + 1]))
    return pairs


def check_format(file_name, encoding):
    file = codecs.open(file_name, encoding)
    for index, line in enumerate(file):
        pair = line.split("\t")
        if not len(pair) == 2:
            print("error", file_name)
            print(line, index, len(pair))
            break

    file.close()


def format_refine(file_name, encoding):
    file = codecs.open(file_name, encoding)
    valid_lines = []
    for index, line in enumerate(file):
        pair = line.split("\t")
        if len(pair) == 2:
            valid_lines.append(line)
    file.close()

    file = codecs.open(file_name, "w", encoding)
    for line in valid_lines:
        file.write(line)
    file.close()
